# -*- coding: utf-8 -*-
from django.utils.translation import gettext_lazy as _
from rest_framework import exceptions, viewsets
from rest_framework.decorators import action
from rest_framework.response import Response
from thx.api.mixins import ThuxModelMixin

from ... import models
from . import serializers

STATUS_CODES = ((0, _('disable')), (1, _('enable')),)


# OFFICE
class OfficeViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    OfficeViewSet
    -----------

    :action list: Return the office list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given office.
        `Profiles`: `superuser`, `staff`

    :action light: Return the office list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new office instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a office instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a office instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a office instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Office.objects.all()
    serializer_class = serializers.OfficeSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'Office{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a office instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a office instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# AREA
class AreaViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    AreaViewSet
    -----------

    :action list: Return the area list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given area.
        `Profiles`: `superuser`, `staff`

    :action light: Return the area list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new area instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a area instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a area instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a area instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Area.objects.all()
    serializer_class = serializers.AreaSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'Area{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a area instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a area instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)
